@@ -79,16 +79,16 @@
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
-#include <inttypes.h>
 
 #include "nasm.h"
 #include "nasmlib.h"
+#include "error.h"
 #include "saa.h"
 #include "stdscan.h"
 #include "labels.h"
 #include "eval.h"
-#include "output/outform.h"
-#include "output/outlib.h"
+#include "outform.h"
+#include "outlib.h"
 
 #ifdef OF_BIN
 
@@ -220,7 +220,7 @@ static struct Section *create_section(char *name)
     return last_section;
 }
 
-static void bin_cleanup(int debuginfo)
+static void bin_cleanup(void)
 {
     struct Section *g, **gp;
     struct Section *gs = NULL, **gsp;
@@ -232,8 +232,6 @@ static void bin_cleanup(int debuginfo)
     uint64_t pend;
     int h;
 
-    (void)debuginfo;      /* placate optimizers */
-
 #ifdef DEBUG
     nasm_error(ERR_DEBUG,
             "bin_cleanup: Sections were initially referenced in this order:\n");
@@ -270,7 +268,7 @@ static void bin_cleanup(int debuginfo)
         if (s->flags & (START_DEFINED | ALIGN_DEFINED | FOLLOWS_DEFINED)) {     /* Check for a mixture of real and virtual section attributes. */
             if (s->flags & (VSTART_DEFINED | VALIGN_DEFINED |
 			    VFOLLOWS_DEFINED))
-                nasm_error(ERR_FATAL|ERR_NOFILE,
+                nasm_fatal(ERR_NOFILE,
                       "cannot mix real and virtual attributes"
                       " in nobits section (%s)", s->name);
             /* Real and virtual attributes mean the same thing for nobits sections. */
@@ -341,11 +339,11 @@ static void bin_cleanup(int debuginfo)
              s && strcmp(s->name, g->follows);
              sp = &s->next, s = s->next) ;
         if (!s)
-            nasm_error(ERR_FATAL|ERR_NOFILE, "section %s follows an invalid or"
+            nasm_fatal(ERR_NOFILE, "section %s follows an invalid or"
                   " unknown section (%s)", g->name, g->follows);
         if (s->next && (s->next->flags & FOLLOWS_DEFINED) &&
             !strcmp(s->name, s->next->follows))
-            nasm_error(ERR_FATAL|ERR_NOFILE, "sections %s and %s can't both follow"
+            nasm_fatal(ERR_NOFILE, "sections %s and %s can't both follow"
                   " section %s", g->name, s->next->name, s->name);
         /* Find the end of the current follows group (gs). */
         for (gsp = &g->next, gs = g->next;
@@ -389,7 +387,7 @@ static void bin_cleanup(int debuginfo)
 	if (sections->flags & START_DEFINED) {
             /* Make sure this section doesn't begin before the origin. */
             if (sections->start < origin)
-                nasm_error(ERR_FATAL|ERR_NOFILE, "section %s begins"
+                nasm_fatal(ERR_NOFILE, "section %s begins"
                       " before program origin", sections->name);
 	} else if (sections->flags & ALIGN_DEFINED) {
             sections->start = ALIGN(origin, sections->align);
@@ -445,13 +443,13 @@ static void bin_cleanup(int debuginfo)
         /* Check for section overlap. */
         if (s) {
 	    if (s->start < origin)
-		nasm_error(ERR_FATAL|ERR_NOFILE, "section %s beings before program origin",
+		nasm_fatal(ERR_NOFILE, "section %s beings before program origin",
 		      s->name);
 	    if (g->start > s->start)
-                nasm_error(ERR_FATAL|ERR_NOFILE, "sections %s ~ %s and %s overlap!",
+                nasm_fatal(ERR_NOFILE, "sections %s ~ %s and %s overlap!",
                       gs->name, g->name, s->name);
             if (pend > s->start)
-                nasm_error(ERR_FATAL|ERR_NOFILE, "sections %s and %s overlap!",
+                nasm_fatal(ERR_NOFILE, "sections %s and %s overlap!",
                       g->name, s->name);
         }
         /* Remember this section as the latest >0 length section. */
@@ -480,7 +478,7 @@ static void bin_cleanup(int debuginfo)
                 for (s = sections; s && strcmp(g->vfollows, s->name);
                      s = s->next) ;
                 if (!s)
-                    nasm_error(ERR_FATAL|ERR_NOFILE,
+                    nasm_fatal(ERR_NOFILE,
                           "section %s vfollows unknown section (%s)",
                           g->name, g->vfollows);
             } else if (g->prev != NULL)
@@ -519,7 +517,7 @@ static void bin_cleanup(int debuginfo)
         }
     }
     if (h)
-        nasm_error(ERR_FATAL|ERR_NOFILE, "circular vfollows path detected");
+        nasm_fatal(ERR_NOFILE, "circular vfollows path detected");
 
 #ifdef DEBUG
     nasm_error(ERR_DEBUG,
@@ -542,6 +540,8 @@ static void bin_cleanup(int debuginfo)
 
         nasm_assert(r->bytes <= 8);
 
+        memset(mydata, 0, sizeof(mydata));
+
         saa_fread(r->target->contents, r->posn, mydata, r->bytes);
         p = mydata;
         l = 0;
@@ -746,7 +746,7 @@ static void bin_out(int32_t segto, const void *data,
     /* Find the segment we are targeting. */
     s = find_section_by_index(segto);
     if (!s)
-        nasm_error(ERR_PANIC, "code directed to nonexistent segment?");
+        nasm_panic(0, "code directed to nonexistent segment?");
 
     /* "Smart" section-type adaptation code. */
     if (!(s->flags & TYPE_DEFINED)) {
@@ -763,7 +763,7 @@ static void bin_out(int32_t segto, const void *data,
     switch (type) {
     case OUT_ADDRESS:
     {
-        int asize = abs(size);
+        int asize = abs((int)size);
 
         if (segment != NO_SEG && !find_section_by_index(segment)) {
             if (segment % 2)
@@ -988,7 +988,7 @@ static int bin_read_attribute(char **line, int *attribute,
     stdscan_reset();
     stdscan_set(exp);
     tokval.t_type = TOKEN_INVALID;
-    e = evaluate(stdscan, NULL, &tokval, NULL, 1, nasm_error, NULL);
+    e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
     if (e) {
         if (!is_really_simple(e)) {
             nasm_error(ERR_NONFATAL, "section attribute value must be"
@@ -1285,7 +1285,8 @@ static int32_t bin_secname(char *name, int pass, int *bits)
     return sec->vstart_index;
 }
 
-static int bin_directive(enum directives directive, char *args, int pass)
+static enum directive_result
+bin_directive(enum directive directive, char *args, int pass)
 {
     switch (directive) {
     case D_ORG:
@@ -1297,7 +1298,7 @@ static int bin_directive(enum directives directive, char *args, int pass)
         stdscan_reset();
         stdscan_set(args);
         tokval.t_type = TOKEN_INVALID;
-        e = evaluate(stdscan, NULL, &tokval, NULL, 1, nasm_error, NULL);
+        e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
         if (e) {
             if (!is_really_simple(e))
                 nasm_error(ERR_NONFATAL, "org value must be a critical"
@@ -1315,7 +1316,7 @@ static int bin_directive(enum directives directive, char *args, int pass)
         } else
             nasm_error(ERR_NONFATAL, "No or invalid offset specified"
                   " in ORG directive.");
-        return 1;
+        return DIRR_OK;
     }
     case D_MAP:
     {
@@ -1324,7 +1325,7 @@ static int bin_directive(enum directives directive, char *args, int pass)
 	char *p;
 	
         if (pass != 1)
-            return 1;
+            return DIRR_OK;
         args += strspn(args, " \t");
         while (*args) {
             p = args;
@@ -1348,12 +1349,12 @@ static int bin_directive(enum directives directive, char *args, int pass)
                 else if (!nasm_stricmp(p, "stderr"))
                     rf = stderr;
                 else {          /* Must be a filename. */
-                    rf = fopen(p, "wt");
+                    rf = nasm_open_write(p, NF_TEXT);
                     if (!rf) {
                         nasm_error(ERR_WARNING, "unable to open map file `%s'",
                               p);
                         map_control = 0;
-                        return 1;
+                        return DIRR_OK;
                     }
                 }
             } else
@@ -1363,10 +1364,10 @@ static int bin_directive(enum directives directive, char *args, int pass)
             map_control |= MAP_ORIGIN | MAP_SUMMARY;
         if (!rf)
             rf = stdout;
-        return 1;
+        return DIRR_OK;
     }
     default:
-	return 0;
+	return DIRR_UNKNOWN;
     }
 }
 
@@ -1396,14 +1397,7 @@ static int32_t bin_segbase(int32_t segment)
     return segment;
 }
 
-static int bin_set_info(enum geninfo type, char **val)
-{
-    (void)type;
-    (void)val;
-    return 0;
-}
-
-struct ofmt of_bin, of_ith, of_srec;
+const struct ofmt of_bin, of_ith, of_srec;
 static void binfmt_init(void);
 static void do_output_bin(void);
 static void do_output_ith(void);
@@ -1429,7 +1423,6 @@ static void srec_init(void)
 
 static void binfmt_init(void)
 {
-    maxbits = 64;               /* Support 64-bit Segments */
     relocs = NULL;
     reloctail = &relocs;
     origin_defined = 0;
@@ -1654,15 +1647,16 @@ static void do_output_srec(void)
 }
 
 
-struct ofmt of_bin = {
+const struct ofmt of_bin = {
     "flat-form binary files (e.g. DOS .COM, .SYS)",
     "bin",
     0,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
     bin_init,
-    bin_set_info,
+    nasm_do_legacy_output,
     bin_out,
     bin_deflabel,
     bin_secname,
@@ -1670,18 +1664,20 @@ struct ofmt of_bin = {
     bin_segbase,
     bin_directive,
     bin_filename,
-    bin_cleanup
+    bin_cleanup,
+    NULL                        /* pragma list */
 };
 
-struct ofmt of_ith = {
+const struct ofmt of_ith = {
     "Intel hex",
     "ith",
     OFMT_TEXT,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
     ith_init,
-    bin_set_info,
+    nasm_do_legacy_output,
     bin_out,
     bin_deflabel,
     bin_secname,
@@ -1689,18 +1685,20 @@ struct ofmt of_ith = {
     bin_segbase,
     bin_directive,
     ith_filename,
-    bin_cleanup
+    bin_cleanup,
+    NULL                        /* pragma list */
 };
 
-struct ofmt of_srec = {
+const struct ofmt of_srec = {
     "Motorola S-records",
     "srec",
-    0,
+    OFMT_TEXT,
+    64,
     null_debug_arr,
     &null_debug_form,
     bin_stdmac,
     srec_init,
-    bin_set_info,
+    nasm_do_legacy_output,
     bin_out,
     bin_deflabel,
     bin_secname,
@@ -1708,7 +1706,8 @@ struct ofmt of_srec = {
     bin_segbase,
     bin_directive,
     srec_filename,
-    bin_cleanup
+    bin_cleanup,
+    NULL                        /* pragma list */
 };
 
 #endif                          /* #ifdef OF_BIN */
